#include "stock.h"

#include <gtk/gtk.h>

int server;
int server2;
int cash;

pthread_mutex_t result_area_mut = PTHREAD_MUTEX_INITIALIZER;

GList *buy = NULL;
GList *sell = NULL;

GtkWidget *combo_buy, *combo_sell;
GtkWidget *entry_buy_qty, *entry_buy_pr;
GtkWidget *entry_sell_qty, *entry_sell_pr;
GtkWidget *text;
GtkWidget *clist_results;
GtkWidget *label_cash;

static void buy_stocks ();
static void sell_stocks ();
static void initialise_ui (int ac, char *av[]);
static void show_qty ();
static void get_last_trade ();
static void get_buy_list ();
static void get_sell_list ();

inline static cid_t get_comp_cid (char *name) {
   cid_t cid;
   
   for (cid=0; cid<=max_companies; ++cid) 
     if (strcmp (company[cid].name, name) == 0)
       break;
   
   return cid;
}

static void get_buy_list () {
   char *name;
   cid_t cid;
   
   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_buy)->entry));
   cid = get_comp_cid (name);
 
   get_comp_list (cid, BUY);
   
}

static void get_sell_list () {
   char *name;
   cid_t cid;
   
   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_buy)->entry));
   cid = get_comp_cid (name);
 
   get_comp_list (cid, SELL);
   
}

static void get_last_trade () {
   char *name;
   cid_t cid;
   
   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_buy)->entry));
   cid = get_comp_cid (name);
 
   get_comp_last_trade (cid);
   return;
}

static void show_qty () {
   char *name;
   cid_t cid;
   char *s;
   
   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_sell)->entry));
   cid = get_comp_cid (name);
   
   s = (char *) malloc (8);
   sprintf (s, "%d", company[cid].qty);
   
   display_results ("Inventory", cid, s, NULL);
   gtk_entry_set_text (GTK_ENTRY (entry_sell_qty), s);
}

static void buy_stocks () {
   gchar *b_qty, *b_pr, *name;
   register cid_t cid;

   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_buy)->entry));
   b_qty = gtk_entry_get_text (GTK_ENTRY (entry_buy_qty));
   b_pr = gtk_entry_get_text (GTK_ENTRY (entry_buy_pr));
   
   cid = get_comp_cid (name);
   
   quote (cid, atoi (b_pr), atoi (b_qty), POST_BUY_QUOT);
   printf ("Buy : %d %d %d\n", cid, atoi (b_pr), atoi (b_qty));
   return;
}

static void sell_stocks () {
   gchar *s_qty, *s_pr, *name;
   register cid_t cid;
   
   name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_sell)->entry));
   s_qty = gtk_entry_get_text (GTK_ENTRY (entry_sell_qty));
   s_pr =  gtk_entry_get_text (GTK_ENTRY (entry_sell_pr));
   
   cid = get_comp_cid (name);
   
   quote (cid, atoi (s_pr), atoi (s_qty), POST_SELL_QUOT);
   printf ("Sell : %d %d %d\n", cid, atoi(s_pr), atoi (s_qty));
   return;
}

void update_result_area (char *s[]) {
   
   pthread_mutex_lock (&result_area_mut);
   gtk_clist_append (GTK_CLIST (clist_results), s);
   pthread_mutex_unlock (&result_area_mut);
}


/* make_box (...) makes the boxes for the Buy and Sell stuff ... */

static GtkWidget *make_trade_box (GtkWidget *vbox, GtkWidget *combo,
				  GtkWidget *entry1, GtkWidget *entry2,
				  GList *list, 
				  void (*button_callback) (void),
				  void (*button_callback2) (void),
				  char *s1, char *s2, char *s3, char *s4,
				  char *s5) {
   GtkWidget *label;
   GtkWidget *button;
      
   vbox = gtk_vbox_new (FALSE, 5);
   
   label = gtk_label_new (s1);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
   gtk_widget_show (label);

   gtk_combo_set_popdown_strings (GTK_COMBO (combo), list);
   gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 5);
   gtk_widget_show (combo);
   
   label = gtk_label_new (s2);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
   gtk_widget_show (label);
   
   gtk_box_pack_start (GTK_BOX (vbox), entry1, FALSE, FALSE, 5);
   gtk_widget_show (entry1);

   label = gtk_label_new (s3);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
   gtk_widget_show (label);
   
   gtk_box_pack_start (GTK_BOX (vbox), entry2, FALSE, FALSE, 5);
   gtk_widget_show (entry2);
   
   button = gtk_button_new_with_label (s4);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 5);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (button_callback), NULL);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label (s5);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 5);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (button_callback2), NULL);
   gtk_widget_show (button);
   
   return vbox;
}

inline static void add_box (GtkWidget *box1, GtkWidget *box2, GtkWidget *pack,
			    gint expand_box1, gint expand_box2) {

   gtk_box_pack_start (GTK_BOX (box2), pack, expand_box1, TRUE, 5);
   gtk_box_pack_start (GTK_BOX (box1), box2, expand_box2, TRUE, 5);
   gtk_widget_show (pack);
   gtk_widget_show (box2);
}

static void initialise_ui (int ac, char *av[]) {
   GtkWidget *window_main;     /* Main Window */
   GtkWidget *vbox_main;
   GtkWidget *hbox, *vbox;
   GtkWidget *button;
   GtkWidget *scrolled_window;
   char *clist_titles[] = {"Transaction", "Company", "Qty", "Pr" };
   
   gtk_init (&ac, &av);
   
   window_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (window_main), "Stock Client");
   gtk_signal_connect (GTK_OBJECT (window_main), "delete_event",
		       quit, NULL);
   
   vbox_main = gtk_vbox_new (FALSE, 5); /* vbox_main is used to pack all hboxes in */
   gtk_widget_show (vbox_main);
   
   hbox = gtk_hbox_new (FALSE, 5);
   
   combo_buy = gtk_combo_new ();
   entry_buy_qty = gtk_entry_new_with_max_length (6);
   entry_buy_pr = gtk_entry_new_with_max_length (6);
   vbox = make_trade_box (vbox, combo_buy, entry_buy_qty, 
			  entry_buy_pr, buy, buy_stocks, get_last_trade,
			  "Buy Company", "Buy Qty", "Buy Price", "Buy",
			  "Last Trade");
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, FALSE, 5);
   gtk_widget_show (vbox);
   
   combo_sell = gtk_combo_new ();
   entry_sell_qty = gtk_entry_new_with_max_length (6);
   entry_sell_pr = gtk_entry_new_with_max_length (6);
   vbox = make_trade_box (vbox, combo_sell, entry_sell_qty, 
			  entry_sell_pr, sell, sell_stocks, show_qty,
			  "Sell Company", "Sell Qty", "Sell Price", "Sell",
			  "Qty Held");
   add_box (vbox_main, hbox, vbox, TRUE, FALSE);
      
   hbox = gtk_hbox_new (FALSE, 5);
   
   label_cash = gtk_label_new (NULL);
   update_cash ();
   add_box (vbox_main, hbox, label_cash, TRUE, FALSE);

   hbox = gtk_hbox_new (FALSE, 5);
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   clist_results = gtk_clist_new_with_titles (4, clist_titles);
   gtk_widget_show (clist_results);
   gtk_clist_set_column_width (GTK_CLIST (clist_results), 0, 75);
   gtk_clist_set_column_width (GTK_CLIST (clist_results), 1, 150);
   gtk_clist_set_column_width (GTK_CLIST (clist_results), 2, 50);
   gtk_clist_set_column_width (GTK_CLIST (clist_results), 3, 50);
 
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window),
					  clist_results);
   add_box (vbox_main, hbox, scrolled_window, TRUE, TRUE);
   
   hbox = gtk_hbox_new (FALSE, 5);

   button = gtk_button_new_with_label ("Get Buy List");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (get_buy_list), NULL);
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);
   gtk_widget_show (button);
   
   
   button = gtk_button_new_with_label ("Quit");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (quit), NULL);
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label ("Get Sell List");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (get_sell_list), NULL);
      
   add_box (vbox_main, hbox, button, TRUE, FALSE);
   

   gtk_container_add (GTK_CONTAINER (window_main), vbox_main);
   gtk_widget_show (window_main);
   
   gtk_main ();
}

void ui_gtk_entry (int ac, char *av[]) {
   
   get_comp_misc ();
   get_comp_names ();
      
   initialise_ui (ac, av);
   return;
}

void update_buy_list (char *name) {
   
   buy = g_list_insert_sorted (buy, name, (GCompareFunc) strcmp);

}

void update_new_buy (cid_t cid, qty_t qty) {
   company_t *c=&company[cid];
   
   c->qty += qty;
   
   if (g_list_find (sell, c->name) == NULL) 
     sell = g_list_insert_sorted (sell, c->name, (GCompareFunc) strcmp);
   gtk_combo_set_popdown_strings (GTK_COMBO (combo_sell), sell);
}

void update_new_sell (cid_t cid, qty_t qty) {
   company_t *c=&company[cid];

   c->qty -= qty;
   
   if (c->qty == 0)
     sell = g_list_remove (sell, c->name);
   gtk_combo_set_popdown_strings (GTK_COMBO (combo_sell), sell);   
}
   
void quit () {
   req_t req=LOGOUT;
   
   sw_cid (req);
   gtk_main_quit();
   exit (0);
}

void update_cash () {
   char *s;
   
   s = (char *) malloc (16);
   sprintf (s, "Cash : %d", cash);
   gtk_label_set_text (GTK_LABEL (label_cash), s);
   free (s);   
}
